<?php
if (!class_exists('Voya_Despachos_Calculo')) {
  class Voya_Despachos_Calculo extends WC_Shipping_Method{
    private $wunit;
    private $dunit;

    public function __construct(){
      //Weight units: kg,g,lbs,oz
      //Dimension units: m,cm,mm,in,yd
      $this->id = 'voya_despachos';
      $this->method_title = __('Voya Chile - Cotizador de Despachos', 'voya_despachos');
      $this->method_description = __('Con este plugin tus clientes podrán seleccionar la empresa de despachos a utilizar y pagar de inmediato el valor del despacho. Este plugin es compatible con los destinos de "MkRapel Regiones y Ciudades de Chile para WC".', 'voya_despachos');
      $this->availability = 'including';
      $this->countries = array(
          'CL'
      );
      $this->wunit = get_option("woocommerce_weight_unit");
      $this->dunit = get_option("woocommerce_dimension_unit");
      
      $this->init();
      $this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
      $this->test_mode = isset($this->settings['test_mode']) ? $this->settings['test_mode'] : 'no';
      $this->api = isset($this->settings['api']) ? $this->settings['api'] : __('', 'voya_despachos');
      $this->default_weight = isset($this->settings['default_weight']) ? $this->settings['default_weight'] : __('1', 'voya_despachos');
      $this->default_height = isset($this->settings['default_height']) ? $this->settings['default_height'] : __('10', 'voya_despachos');
      $this->default_width = isset($this->settings['default_width']) ? $this->settings['default_width'] : __('10', 'voya_despachos');
      $this->default_length = isset($this->settings['default_length']) ? $this->settings['default_length'] : __('10', 'voya_despachos');
      $this->default_percentage = isset($this->settings['default_percentage']) ? $this->settings['default_percentage'] : __('0', 'voya_despachos');

      $this->enable_default_shipping = isset($this->settings['enable_default_shipping']) ? $this->settings['enable_default_shipping'] : '';

      $this->free_shipping_threshold = isset($this->settings['free_shipping_threshold']) ? $this->settings['free_shipping_threshold'] : 0;
      $this->free_shipping_destination = isset($this->settings['free_shipping_destination']) ? $this->settings['free_shipping_destination'] : "";
      $this->free_shipping_date_limit = isset($this->settings['free_shipping_date_limit']) ? $this->settings['free_shipping_date_limit'] : "";
    }

    function init(){
      $this->init_form_fields();
      $this->init_settings();
      add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
    }

    function init_form_fields(){
      include( VOYA_PLUGIN_PATH . '/includes/data/cities/CL_VOYA_FS_CITIES.php' );
      $this->form_fields = array(
          'enabled' => array(
              'title' => __('Activo', 'voya_despachos'),
              'type' => 'checkbox',
              'label' => 'Activo',
              'description' => __('Activar Cotizador de despachos Voya', 'voya_despachos'),
              'default' => 'yes'
          ),
          'api' => array(
              'title' => __('API KEY', 'voya_despachos'),
              'type' => 'text',
              'description' => __('Las consultas de valores son procesadas a través de nuestros servidores. Regístrate en <a href="https://app.voyapp.cl/register" target="_blank"><strong>Voya</strong></a> para obtener una API KEY para tu dominio.</br> Si quieres conocer nuestros planes, visita la página del plugin haciendo <a href="https://voyapp.cl/planes" target="_blank">click aquí.</a>', 'voya_despachos'),
              'default' => __('', 'voya_despachos')
          ),
          'test_mode' => array(
              'title' => __('¿Modo de pruebas?', 'voya_despachos'),
              'type' => 'checkbox',
              'label' => 'Activo',
              'description' => __('Activa esta opción si estás recién desarrollando tu ecommerce o estás realizando las pruebas de integración. Las consultas realizadas bajo esta modalidad no serán consideradas en tu facturación mensual.', 'voya_despachos'),
              'default' => 'yes'
          ),
          'default_weight' => array(
              'title' => __('Peso por defecto del producto', 'voya_despachos'),
              'type' => 'number',
              'description' => __('Este valor será utilizado en los productos que no tengan un "peso" definido en la tienda. Ingresar valor en <strong>' . $this->wunit . '</strong>.', 'voya_despachos'),
              'default' => __('1', 'voya_despachos')
          ),
          'default_height' => array(
              'title' => __('Altura por defecto del producto', 'voya_despachos'),
              'type' => 'number',
              'description' => __('Este valor será utilizado en los productos que no tengan una "altura" definida en la tienda. Ingresar valor en <strong>' . $this->dunit . '</strong>.', 'voya_despachos'),
              'default' => __('10', 'voya_despachos')
          ),
          'default_width' => array(
              'title' => __('Ancho por defecto del producto', 'voya_despachos'),
              'type' => 'number',
              'description' => __('Este valor será utilizado en los productos que no tengan un "ancho" definido en la tienda. Ingresar valor en <strong>' . $this->dunit . '</strong>.', 'voya_despachos'),
              'default' => __('10', 'voya_despachos')
          ),
          'default_length' => array(
              'title' => __('Largo por defecto del producto', 'voya_despachos'),
              'type' => 'number',
              'description' => __('Este valor será utilizado en los productos que no tengan un "largo" definido en la tienda. Ingresar valor en <strong>' . $this->dunit . '</strong>.', 'voya_despachos'),
              'default' => __('10', 'voya_despachos')
          ),
          'default_percentage' => array(
              'title' => __('Variación porcentual', 'voya_despachos'),
              'type' => 'number',
              'description' => __('Este campo te permite aumentar/disminuir (en porcentaje) el valor final de los envíos. <br>Ejemplo 1: Si quieres que los precios de los despachos aumenten en un 10%, debes ingresar el valor <b>10</b> en este campo.<br>Ejemplo 2: Si quieres que los precios de los despachos disminuyan en un 5%, debes ingresar el valor <b>-5</b> en este campo.', 'voya_despachos'),
              'default' => __('0', 'voya_despachos')
          ),
          'enable_default_shipping' => array(
              'title' => __('¿Habilitar modalidad de despacho "Envío por pagar"?', 'voya_despachos'),
              'type' => 'checkbox',
              'label' => 'Activo',
              'description' => __('Activa esta opción si deseas incluir la modalidad de despacho "Envío por pagar" en caso de que no exista una respuesta de cotización desde los servidores de los couriers.', 'voya_despachos'),
              'default' => ''
          ),
          'free_shipping_threshold' => array(
              'title' => 'Envío gratis según total de compra del cliente',
              'type' => 'number',
              'description' => 'Este campo permite habilitar la modalidad de despacho "Envío Gratis" cuando el precio total del pedido de un comprador supera el valor especificado en este campo. Si no quieres habilitar el envío gratis mediante esta modalidad, deja este campo vacío.',
              'default' => __('0', 'voya_despachos')
          ),
          'free_shipping_destination' => array(
              'title' => 'Envío gratis según destino de envío del cliente',
              'type' => 'multiselect',
              'options' => $fsCities,
              'description' => 'Este campo permite habilitar la modalidad de despacho "Envío Gratis" cuando la comuna de despacho seleccionada por el comprador se encuentra seleccionada en esta lista. Si no quieres habilitar el envío gratis mediante esta modalidad, deja este campo vacío.',
          ),
          'free_shipping_date_limit' => array(
              'title' => 'Fecha de finalización de "Envío Gratis"',
              'type' => 'text',
              'css' => 'background-color: white;',
              'description' => 'Este campo permite establecer la fecha de finalización del "Envío Gratis" de las 2 opciones anteriores. El "Envío Gratis" se desactivará a las 23:59 de la fecha seleccionada.<br>Si no quieres establecer una fecha límite, deja este campo vacío. Esta opción utiliza la zona horaria establecida en la sección de WordPress "Ajustes" -> "Generales" -> "Zona horaria".',
              'default' => ''
          ), 
        );
    }
    
    function admin_options() {
      ?>
        <h2>Voya Chile - Cotizador de Despachos</h2>
        <p>Con este plugin tus clientes podrán seleccionar la empresa de despachos a utilizar y pagar de inmediato el valor del despacho. Este plugin es compatible con los destinos de "MkRapel Regiones y Ciudades de Chile para WC".</p>
        <table class="form-table">
        <?php $this->generate_settings_html(); ?>
        </table>
        <style>
          button.ui-datepicker-current{
            display: none;
          }
        </style>
        <script>
          jQuery(document).ready(function() {
              jQuery('#woocommerce_voya_despachos_free_shipping_destination').select2();
              jQuery('#woocommerce_voya_despachos_free_shipping_date_limit').datepicker({
                minDate: new Date(),
                showOn: "button",
                buttonText: "Seleccionar fecha",
                showButtonPanel: true,
                prevText: '<Ant',
                nextText: 'Sig>',
                currentText: 'Hoy',
                monthNames: ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio',
                'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'],
                monthNamesShort: ['Ene', 'Feb', 'Mar', 'Abr', 'May', 'Jun',
                'Jul', 'Ago', 'Sep', 'Oct', 'Nov', 'Dic'],
                dayNames: ['Domingo', 'Lunes', 'Martes', 'Miércoles', 'Jueves', 'Viernes', 'Sábado'],
                dayNamesShort: ['Dom', 'Lun', 'Mar', 'Mié;', 'Juv', 'Vie', 'Sáb'],
                dayNamesMin: ['Do', 'Lu', 'Ma', 'Mi', 'Ju', 'Vi', 'Sá'],
                weekHeader: 'Sm',
                dateFormat: 'dd/mm/yy',
                firstDay: 1,
                isRTL: false,
                showMonthAfterYear: false,
                yearSuffix: '',
                closeText: 'Vaciar campo',
                onClose: function () {
                    var event = arguments.callee.caller.caller.arguments[0];                
                    if (event && jQuery(event.delegateTarget).hasClass('ui-datepicker-close')) {
                        jQuery(this).val('');
                    }
                }
              }).attr("readonly", true).next(".ui-datepicker-trigger").addClass("button");
          });

        </script>
      <?php
    }
    
    public function calculate_shipping($package = array()){
      try{
        if((!$package['destination']['city'] || !$package['destination']['address']) || $this->settings['api'] == ""){
          $this->write_log("ERROR: NO SE EJECUTARÁ LA CONSULTA PORQUE NO ESTÁ ESTABLECIDA LA APIKEY O NO SE HA SELECCIONADO UN DESTINO + DIRECCIÓN.");
          return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
        }
        //Envíos gratis  
        if($this->freeShipping($package)){
          return;
        }

        $headers = $this->getRequestHeaders();
        $data = $this->getRequestInitialData($package);
        if(!$data){
          $this->write_log("ERROR: NO SE EJECUTARÁ LA CONSULTA PORQUE LA CIUDAD SOLICITADA NO ESTÁ MAPEADA.");
          $this->checkAndAddDefaultShipping();
          return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
        }
        $apiUrl = "";
        if($this->test_mode == 'yes'){
          $apiUrl = $this->getApiUrl("test");
        }else{
          $apiUrl = $this->getApiUrl("tarifas");
        }
        $weight = 0;
        $finalPackage = array(0,0,0);
        $packagePerProduct = array();
        $packageQty = count($package["contents"]);
        $counter = 0;
        foreach ($package['contents'] as $item_id => $values){
          $_product = $values['data'];
          $productDimensions = $this->validateAndGetDimensions($_product);
          $weight += $productDimensions["weight"] * $values['quantity'];
          $packagePerProduct[$counter] = array(0,$productDimensions["length"],$productDimensions["width"],$productDimensions["height"]);
          sort($packagePerProduct[$counter]);
          $packagePerProduct[$counter][1] = $packagePerProduct[$counter][1] * $values['quantity'];
          sort($packagePerProduct[$counter]);
          $packagePerProduct[$counter][0] = $packagePerProduct[$counter][1] * $packagePerProduct[$counter][2] * $packagePerProduct[$counter][3];
          $counter++;
        }
        usort($packagePerProduct, function($a, $b) {
          return $b[0] - $a[0];
        });

        for ($counter=0; $counter < $packageQty; $counter++) {
          $finalPackage[0] = $finalPackage[0] + $packagePerProduct[$counter][1];
          if ($packagePerProduct[$counter][2] > $finalPackage[1]) {
            $finalPackage[1] = $packagePerProduct[$counter][2];
          }
          if ($packagePerProduct[$counter][3] > $finalPackage[2]) {
            $finalPackage[2] = $packagePerProduct[$counter][3];
          }
          sort($finalPackage);
        }
        $data["height"] = ceil(wc_get_dimension($finalPackage[0],'cm',$this->dunit));
        $data["width"] = ceil(wc_get_dimension($finalPackage[1],'cm',$this->dunit));
        $data["length"] = ceil(wc_get_dimension($finalPackage[2],'cm',$this->dunit));
        $data["volume"] = $data["height"] * $data["width"] *  $data["length"];
        $data["weight"] = wc_get_weight($weight, 'kg', $this->wunit);
        $result = wp_remote_post($apiUrl, array(
              'headers' => $headers,
              'method' => 'POST',
              'body' => http_build_query($data),
              'timeout' => 13
            )
          );

        if (is_wp_error($result)) {
          $this->write_log("ERROR IS_WP_ERROR");
          $this->write_log($result);
          $this->checkAndAddDefaultShipping();
          return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
        }

        $decoded = json_decode($result['body']);
        if (isset($decoded->status) && $decoded->status == 'ERROR') {
          $this->write_log("ERROR VOYA - MENSAJE INFORMATIVO:");
          $this->write_log($decoded->message);
          $this->checkAndAddDefaultShipping();
          return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
        }
        if(isset($decoded->message) && $decoded->message != "OK"){
          switch ($decoded->message) {
            case 'Unauthenticated.':
              $this->write_log("ERROR: PROBLEMA DE AUTENTICACIÓN EN EL SISTEMA VOYA.");
              break;
            case 'Too Many Attempts.':
              $this->write_log("ERROR: DEMASIADAS CONSULTAS CONCURRENTES.");
              break;
            
            default:
               $this->write_log("ERROR: ".$decoded->message);
              break;
          }
          $this->checkAndAddDefaultShipping();
          return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
        }
        if (isset($decoded->couriers) && count($decoded->couriers)>0) {
          foreach ($decoded->couriers as $courier) {
            $priceVariation = ($courier->cost / 100) * (int)$this->default_percentage;
            $rate = array(
              'id' => $courier->id,
              'label' => $courier->title,
              'cost' => (int) ($courier->cost + $priceVariation)
            );
            $this->add_rate($rate);
          }
        }else{
          $this->write_log("INFO VOYA: EL SISTEMA VOYA HA ENTREGADO UNA RESPUESTA VACÍA. ESTO PUEDE DEBERSE A: 1.- DOMINIO INACTIVO. 2.- SIN COURIERS SELECCIONADOS. 3.- LAS APIS EXTERNAS DE LOS COURIERS NO HAN ENTREGADO UNA RESPUESTA.");
          $this->checkAndAddDefaultShipping();
        }
      }catch(Exception $e){
        $this->write_log($e->getMessage());
        $this->checkAndAddDefaultShipping();
        return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', false, $package);
      }
      
    }

    private function validateAndGetDimensions($_product){
      $values = array();
      if($_product->get_weight() != null && !empty($_product->get_weight()) && $_product->get_weight() > 0 ){
        $values["weight"] = $_product->get_weight();
      }else{
        $values["weight"] = $this->default_weight;
      }

      //Valida LARGO isset, empty y >0
      if($_product->get_length() != null && !empty($_product->get_length()) && $_product->get_length() > 0 ){
        $values["length"] = $_product->get_length();
      }else{
        $values["length"] =  $this->default_length;
      }

      //Valida ANCHO isset, empty y >0
      if($_product->get_width()!= null  && !empty($_product->get_width()) && $_product->get_width() > 0 ){
        $values["width"] = $_product->get_width();
      }else{
        $values["width"] = $this->default_width;
      }

      //Valida ALTO isset, empty y >0
      if($_product->get_height()!= null  && !empty($_product->get_height()) && $_product->get_height() > 0 ){
        $values["height"] = $_product->get_height();
      }else{
        $values["height"] = $this->default_height;
      }
      return $values;
    }

    private function getApiUrl($api){
      $url = "https://app.voyapp.cl/api/";
      switch ($api) {
        case 'tarifas':
          $url .= "get-rates";
          break;
        case 'test':
          $url .= "integration";
          break;
        default:
          $url .= "get-rates";
          break;
      }
      return $url;
    }

    private function getRequestHeaders(){
      $headers = array();
      $headers["Authorization"] = "Bearer ".$this->api;
      $headers["Accept"] = "application/json";
      return $headers;
    }

    private function getRequestInitialData($package){
      $destination = $this->mapPlaces($package['destination']['city']);
      if($destination){
        $data = array();
        $data["domain"] = get_site_url();
        $data["platform"] = "wp_wc";
        $data["weight_unit"] = 'kg';
        $data["dimension_unit"] = 'cm';
        $data["destination"] = $destination;
        return $data;
      }else{
        return false;
      }
      
    }

    private function mapPlaces($city){
      if (file_exists( VOYA_PLUGIN_PATH . '/includes/data/cities/CL_VOYA.php' ) ) {
        include( VOYA_PLUGIN_PATH . '/includes/data/cities/CL_VOYA.php' );
      }else{
        $this->write_log("ERROR: ARCHIVO CL_VOYA NO ENCONTRADO.");
        return false;
      }
      if(isset($mapPlaces) &&  array_key_exists($city, $mapPlaces) && isset($mapPlaces[$city])){
        return $mapPlaces[$city];
      }else{
        $this->write_log("ERROR: INDEX DE CIUDAD ".$city." NO ENCONTRADO EN ARCHIVO CL_VOYA.");
        return false;
      }
      
    }

    private function checkAndAddDefaultShipping(){
      if($this->enable_default_shipping == "yes"){
        $rate = array(
            'id' => "VOYA_DEFAULT_SM",
            'label' => "Envío por pagar",
            'cost' => 0
          );
        $this->add_rate($rate);
      }
    }

    private function freeShipping($package){
      $evaluate = true;
      if(isset($this->free_shipping_date_limit)){
        $tz = wp_timezone();
        
        $fechaNow = new DateTime("now",$tz);
        $fechaLimite = DateTime::createFromFormat('d/m/Y H:i:s', $this->free_shipping_date_limit.' 23:59:59',$tz);

        if($fechaNow > $fechaLimite){
          $evaluate = false;
        }        
      }
      if($evaluate){
        //Envío gratis segun total compra
        if(!empty($this->free_shipping_threshold) && $this->free_shipping_threshold > 0 && isset($package["cart_subtotal"]) && $package["cart_subtotal"] > 0 && $this->free_shipping_threshold < $package["cart_subtotal"]){
          $rate = array(
            'id' => "VOYA_FREE_SM",
            'label' => "Envío gratis",
            'cost' => 0
          );
          $this->add_rate($rate);
          return true;
        }
        //Envio gratis por destino
        elseif ($this->free_shipping_destination) {
          // && in_array($package['destination']['city'],$this->free_shipping_destination) 
          $destination = $this->mapPlaces($package['destination']['city']);
          if($destination && in_array($destination,$this->free_shipping_destination)){
            $rate = array(
              'id' => "VOYA_FREE_SM",
              'label' => "Envío gratis",
              'cost' => 0
            );
            $this->add_rate($rate);
            return true;
          }
          return false;
        }
        else{
          return false;
        }
      }else{
        return false;
      }
    }

    private function write_log ( $log )  {
      if ( is_array( $log ) || is_object( $log ) ) {
        error_log( print_r( $log, true ) );
      } else {
        error_log( $log );
      }
    }
  }
}